package com.huaxia.comm.util;

import com.ruoyi.common.utils.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.Row;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * ExcelTool
 *
 * @author sulingfeng
 * @date 2023/8/29 14:16
 */
public class ExcelTool {

    /**
     * 将excel纵坐标自然数序号（从1开始）转为字母字符串纵坐标（从A开始）
     * @created by kangtengjiao 2022-03-05
     *
     * @param colNo excel字母字符串纵坐标
     * @returns {string}
     */
    public static String reconvertExcelCol(int colNo) {
        if(colNo <= 0) {
            return "";
        }
        List<String> a = new ArrayList<>();
        do {
            colNo--;
            int n = colNo % 26;
            char c = (char)(65 + n);
            a.add( String.valueOf( c ) );
            colNo = (colNo - n) / 26;
        } while(colNo > 0);

        Collections.reverse(a);
        return a.stream().reduce((s1,s2)->s1+s2).orElse("");
    }

    /**
     * 将excel字母字符串（从A开始）纵坐标转为自然数序号（从1开始）
     * @created by kangtengjiao 2022-03-04
     *
     * @param colStr excel字母字符串纵坐标
     * @returns {number} 自然数序号
     */
    public static int convertExcelCol(String colStr) {
        final int colLen = colStr.length();
        colStr = colStr.toUpperCase();
        int colNo = 0;
        for(int i = 0; i < colLen; i ++) {
            colNo += ( ((colStr.charAt(i) - 65) + 1) * Math.pow(26,colLen - i - 1) );
        }
        return colNo;
    }

    public static class ExcelPos {
        public final int rowIndex;
        public final int colIndex;

        public ExcelPos(int rowIndex, int colIndex) {
            this.rowIndex = rowIndex;
            this.colIndex = colIndex;
        }
    }

    /**
     * 将Excel单元格坐标转换为下标<p>
     *   eg.例如将字符串A1解析成第一行第一列;A2解析成第二行第一列
     * @created by kangtengjiao 2022-03-04
     *
     * @param excelPos excel字符串单元格坐标，eg：A1
     */
    public static ExcelPos convertExcelPos(String excelPos) {

        final String regInt = "^\\d+$", regWord = "^[A-Z]+$";

        // (行号)整数字符串的开始下标
        int numIndex = 0;

        Pattern pattern = Pattern.compile(regWord, Pattern.CASE_INSENSITIVE);

        final int len = excelPos.length();

        for(int i = 0; i < len; i++) {

            Matcher matcher = pattern.matcher( String.valueOf(excelPos.charAt(i)) );
            boolean matches = matcher.matches();

            if( !matches ) {
                numIndex = i;
                break;
            }
        }

        if( numIndex == 0) {
            System.err.println("无效的convertExcelPos方法参数: " + excelPos);
            return null;
        }

        // 提取行号字符串
        String rowStr = excelPos.substring(numIndex);
        if( rowStr.length() == 0 || !Pattern.matches(regInt, rowStr)) {
            System.err.println("无效的convertExcelPos方法参数: " + excelPos);
            return null;
        }
        // 行号
        final int rowNo = Integer.parseInt(rowStr);

        // 提取列号字符串
        String colStr = excelPos.substring(0, numIndex).toUpperCase();

        // 列号
        final int colNo = convertExcelCol(colStr);

        return new ExcelPos(rowNo - 1, colNo - 1);
    }

    public static void main(String[] args) {
        System.out.println( ExcelTool.reconvertExcelCol(27) );

        System.out.println( ExcelTool.convertExcelCol("Z") );
        final String regInt = "^\\d+$", regWord = "^[A-Z]+$";

        // 测试数字
        boolean a = Pattern.matches(regInt, "10");
        System.out.println(a);

        // 测试字母
        Pattern pattern = Pattern.compile(regWord, Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher("Ag");
        boolean matches = matcher.matches();

        System.out.println(matches);

        //
        System.out.println( convertExcelPos("A2") );

    }

    public static Object getCellValue(Row row, int column) {
        if (row == null) {
            return row;
        } else {
            Object val = "";

            try {
                Cell cell = row.getCell(column);
                if (StringUtils.isNotNull(cell)) {
                    CellType cellType = cell.getCellType();
                    switch (cellType) {
                        case STRING:
                            val = cell.getStringCellValue();
                            break;
                        case NUMERIC:
                            val = cell.getNumericCellValue();
                            if (DateUtil.isCellDateFormatted(cell)) {
                                val = DateUtil.getJavaDate((Double)val);
                            } else if ((Double)val % 1.0 > 0.0) {
                                val = new BigDecimal(val.toString());
                            } else {
                                val = (new DecimalFormat("0")).format(val);
                            }
                            break;
                        case BOOLEAN:
                            val = cell.getBooleanCellValue();
                            break;
                        case FORMULA:
                            switch (cell.getCachedFormulaResultType()) {
                                case STRING:
                                    val = cell.getStringCellValue();
                                    return val;
                                case NUMERIC:
                                    val = cell.getNumericCellValue();
                                    if (DateUtil.isCellDateFormatted(cell)) {
                                        val = DateUtil.getJavaDate((Double)val);
                                        return val;
                                    } else {
                                        if ((Double)val % 1.0 > 0.0) {
                                            val = new BigDecimal(val.toString());
                                        } else {
                                            val = (new DecimalFormat("0")).format(val);
                                        }

                                        return val;
                                    }
                                case BOOLEAN:
                                    val = cell.getBooleanCellValue();
                                    return val;
                                default:
                                    val = cell.getCellFormula();
                                    return val;
                            }
                        case ERROR:
                            val = cell.getErrorCellValue();
                    }
                }

                return val;
            } catch (Exception var5) {
                return val;
            }
        }
    }

}
