package com.huaxia.comm.util;

import com.ruoyi.common.exception.base.BaseException;
import com.ruoyi.common.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ooxml.POIXMLDocumentPart;
import org.apache.poi.openxml4j.util.ZipSecureFile;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.usermodel.Color;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.util.Units;
import org.apache.poi.xssf.usermodel.*;
import org.openxmlformats.schemas.drawingml.x2006.main.*;
import org.openxmlformats.schemas.drawingml.x2006.spreadsheetDrawing.CTConnector;
import org.openxmlformats.schemas.drawingml.x2006.spreadsheetDrawing.CTMarker;
import org.openxmlformats.schemas.drawingml.x2006.spreadsheetDrawing.CTShape;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import cn.hutool.http.HttpUtil;

import java.awt.*;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

import static org.apache.poi.ss.usermodel.CellType.*;

@Slf4j
public class PoiExcelTool {

    /**
     * 301px ≈ 701dx ≈ 144 dy
     * 根据列宽和横坐标像素计算 dx
     * @param px
     * @param colWidth
     * @return 0~1023
     */
    public static int getAnchorX(int px, int colWidth){
        return (int) Math.round(( (double)701*16000.0/301)*((double)1/colWidth)*px);
    }

    /**
     * 301px ≈ 701dx ≈ 144 dy
     * 根据行高和纵坐标像素计算dy
     * @param px
     * @param rowHeight 行高默认8000
     * @return 0~255
     */
    public static int getAnchorY(int px, int rowHeight){
        return (int) Math.round(( (double)144 * 8000/301)*((double)1/rowHeight)*px);
    }

    /**
     * 线条样式常量类
     *
     * @author xudongmaster
     */
    public interface LineStyleConstant {
        int INT_SOLID = 1;
        int INT_DOT = 2;
        int INT_DASH = 3;
        int INT_LG_DASH = 4;
        int INT_DASH_DOT = 5;
        int INT_LG_DASH_DOT = 6;
        int INT_LG_DASH_DOT_DOT = 7;
        int INT_SYS_DASH = 8;
        int INT_SYS_DOT = 9;
        int INT_SYS_DASH_DOT = 10;
        int INT_SYS_DASH_DOT_DOT = 11;
    }

    /**
     * 给Cell添加批注
     *
     * @param cell      单元格
     * @param value     批注内容
     * @param extension 扩展名
     */
    public static void addComment(Cell cell, String value, String extension) {
        Sheet sheet = cell.getSheet();
        cell.removeCellComment();
        if ("xls".equals(extension)) {
            ClientAnchor anchor = new HSSFClientAnchor();
            // 关键修改
            anchor.setDx1(0);
            anchor.setDx2(0);
            anchor.setDy1(0);
            anchor.setDy2(0);
            anchor.setCol1(cell.getColumnIndex());
            anchor.setRow1(cell.getRowIndex());
            anchor.setCol2(cell.getColumnIndex() + 5);
            anchor.setRow2(cell.getRowIndex() + 6);
            // 结束
            Drawing drawing = sheet.getDrawingPatriarch();
            if (drawing == null) {
                drawing = sheet.createDrawingPatriarch();
            }
            Comment comment = drawing.createCellComment(anchor);
            // 输入批注信息
            comment.setString(new HSSFRichTextString(value));
            // 将批注添加到单元格对象中
            cell.setCellComment(comment);
        } else if ("xlsx".equals(extension)) {
            ClientAnchor anchor = new XSSFClientAnchor();
            // 关键修改
            anchor.setDx1(0);
            anchor.setDx2(0);
            anchor.setDy1(0);
            anchor.setDy2(0);
            anchor.setCol1(cell.getColumnIndex());
            anchor.setRow1(cell.getRowIndex());
            anchor.setCol2(cell.getColumnIndex() + 5);
            anchor.setRow2(cell.getRowIndex() + 6);
            // 结束
            Drawing drawing = sheet.getDrawingPatriarch();
            if (drawing == null) {
                drawing = sheet.createDrawingPatriarch();
            }
            Comment comment = drawing.createCellComment(anchor);
            // 输入批注信息
            comment.setString(new XSSFRichTextString(value));
            // 将批注添加到单元格对象中
            cell.setCellComment(comment);
        }
    }

    /**
     * 插入网络图片
     *
     * @param workbook      文档对象
     * @param sheet         sheet页对象
     * @param url           网络路径
     * @param beginRowIndex 开始行号
     * @param endRowIndex   结束行号
     * @param beginColIndex 开始列号
     * @param endColIndex   结束列号
     */
    public static Picture insertUrlImg(Workbook workbook, Sheet sheet, String url, int beginRowIndex, int endRowIndex
            , int beginColIndex, int endColIndex) {
        byte[] bytes = HttpUtil.downloadBytes(url);//hutool的HttpUtil
        return insertImg(workbook, sheet, bytes, beginRowIndex, endRowIndex, beginColIndex, endColIndex);
    }


    /**
     * 插入图片
     *
     * @param workbook      文档对象
     * @param sheet         sheet页对象
     * @param picture       图片二进制流数组
     * @param beginRowIndex 开始行号
     * @param endRowIndex   结束行号
     * @param beginColIndex 开始列号
     * @param endColIndex   结束列号
     */
    public static Picture insertImg(Workbook workbook, Sheet sheet, byte[] picture, int beginRowIndex, int endRowIndex
            , int beginColIndex, int endColIndex) {
        return insertImg(workbook, sheet, picture, 0, 0, 255, 255, beginRowIndex, endRowIndex, beginColIndex, endColIndex);
    }

    /**
     *
     * @param workbook
     * @param sheet
     * @param pictureBytes
     * @param dx1 图片左边界距离单元格左边框像素值
     * @param dy1 图片上边界距离单元格上边框像素值,
     * @param dx2 图片右下角x坐标
     * @param dy2 图片右下角y坐标 坐标x轴总比1023，坐标y轴总比255,图片排列样式根据左上角与右下角坐标比例排列
     * @param beginRowIndex
     * @param endRowIndex
     * @param beginColIndex
     * @param endColIndex
     */
    public static Picture insertImg(Workbook workbook, Sheet sheet, byte[] pictureBytes, int dx1, int dy1, int dx2, int dy2, int beginRowIndex, int endRowIndex
            , int beginColIndex, int endColIndex) {
        //画图的顶级管理器，一个sheet只能获取一个（一定要注意这点）
        Drawing drawing = sheet.getDrawingPatriarch();
        if (drawing == null) {
            drawing = sheet.createDrawingPatriarch();
        }
        //anchor主要用于设置图片的属性
        ClientAnchor anchor = null;
        if (workbook instanceof HSSFWorkbook) {
            anchor = new HSSFClientAnchor(dx1, dy1, dx2, dy2, (short) beginColIndex, beginRowIndex, (short) endColIndex, endRowIndex);
        } else if (workbook instanceof XSSFWorkbook) {
            anchor = new XSSFClientAnchor(dx1, dy1, dx2, dy2, (short) beginColIndex, beginRowIndex, (short) endColIndex, endRowIndex);
        }
        //插入图片
        Picture picture = drawing.createPicture(anchor, workbook.addPicture(pictureBytes, Workbook.PICTURE_TYPE_JPEG));
        return picture;
    }

    /**
     * 替换Excel模板文件内容
     *
     * @param replaceMap 文档数据
     */
    public static void replaceSheet(Map<String, Object> replaceMap, Workbook workbook, Sheet sheet, Integer endRowIndex) {
        try {
            Iterator rows = sheet.rowIterator();
            while (rows.hasNext()) {
                Row row = (Row) rows.next();
                if (row == null) {
                    continue;
                }
                int num = row.getLastCellNum();
                if (endRowIndex != null && row.getRowNum() > endRowIndex) {
                    break;
                }
                for (int i = 0; i < num; i++) {
                    Cell cell = row.getCell(i);
                    if (cell == null || cell.getCellType() == NUMERIC || cell.getStringCellValue() == null) {
                        continue;
                    }
                    String value = cell.getStringCellValue();
                    if (StringUtils.isNotEmpty(value) && value.indexOf("${") != -1) {
                        cell.setCellValue(replaceValue(value, replaceMap));
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 替换内容 ${}
     *
     * @param value
     * @param replaceMap
     * @return
     */
    public static String replaceValue(String value, Map<String, Object> replaceMap) {
        String str = value.substring(value.indexOf("${") + 2, value.indexOf('}'));
        if (StringUtils.isEmpty(str)) {
            return value;
        }
        value = value.replace("${" + str + "}", (String) replaceMap.get(str));
        return value;
    }

    /**
     * 插入线条(线条颜色默认为黑色，线条样式默认为Solid，线条宽度默认为0.5)
     *
     * @param workbook      文档对象
     * @param sheet         sheet页对象
     * @param beginRowIndex 开始行号
     * @param endRowIndex   结束行号
     * @param beginColIndex 开始列号
     * @param endColIndex   结束列号
     */
    public static void insertLine(Workbook workbook, Sheet sheet, int beginRowIndex, int endRowIndex, int beginColIndex
            , int endColIndex) {
        insertLine(workbook, sheet, beginRowIndex, endRowIndex, beginColIndex, endColIndex, getXSSFRGBColor(0, 0, 0)
                , LineStyleConstant.INT_SOLID, 0.5d);
    }

    /**
     * 插入线条
     *
     * @param workbook      文档对象
     * @param sheet         sheet页对象
     * @param beginRowIndex 开始行号
     * @param endRowIndex   结束行号
     * @param beginColIndex 开始列号
     * @param endColIndex   结束列号
     * @param lineColor     线条颜色
     * @param lineStyle     线条样式
     * @param lineWidth     线条宽度
     */
    public static void insertLine(Workbook workbook, Sheet sheet, int beginRowIndex, int endRowIndex, int beginColIndex, int endColIndex
            , Color lineColor, int lineStyle, double lineWidth) {
        insertShape(workbook, sheet, beginRowIndex, endRowIndex, beginColIndex, endColIndex, lineColor, lineStyle, lineWidth
                , ShapeTypes.LINE);
    }

    /**
     * 插入形状
     *
     * @param workbook      文档对象
     * @param sheet         sheet页对象
     * @param beginRowIndex 开始行号
     * @param endRowIndex   结束行号
     * @param beginColIndex 开始列号
     * @param endColIndex   结束列号
     * @param lineColor     线条颜色
     * @param lineStyle     线条样式
     * @param lineWidth     线条宽度
     * @param shapeType     形状类型
     */
    public static void insertShape(Workbook workbook, Sheet sheet, int beginRowIndex, int endRowIndex, int beginColIndex, int endColIndex
            , Color lineColor, int lineStyle, double lineWidth, int shapeType) {
        CreationHelper helper = workbook.getCreationHelper();
        //画图的顶级管理器，一个sheet只能获取一个（一定要注意这点）
        Drawing drawing = sheet.getDrawingPatriarch();
        if (drawing == null) {
            drawing = sheet.createDrawingPatriarch();
        }
        ClientAnchor anchor = helper.createClientAnchor();
        // 设置线条的开始位置
        anchor.setCol1(beginColIndex);
        anchor.setRow1(beginRowIndex);
        // 设置线条的结束位置
        anchor.setCol2(endColIndex);
        anchor.setRow2(endRowIndex);
        if (drawing instanceof XSSFDrawing) {
            XSSFSimpleShape shape = ((XSSFDrawing) drawing).createSimpleShape((XSSFClientAnchor) anchor);
            // 设置形状类型
            shape.setShapeType(shapeType);
            // 设置线宽
            shape.setLineWidth(lineWidth);
            // 设置线的风格
            shape.setLineStyle(lineStyle - 1);
            // 设置线的颜色
            XSSFColor xssfColor = (XSSFColor) lineColor;
            byte[] rgbBytes = xssfColor.getRGB();
            if (rgbBytes == null || rgbBytes.length < 3) {
                shape.setLineStyleColor(0, 0, 0);
            } else {
                shape.setLineStyleColor(rgbBytes[0], rgbBytes[1], rgbBytes[2]);
            }
        }
    }

    /**
     * 行复制功能(单行，不限是否同一个sheet)
     *
     * @param wb     workbook
     * @param oldRow 要复制的行
     * @param toRow  复制到这里行
     */
    public static void copyRow(Workbook wb, Row oldRow, Row toRow) {
        CellStyle newStyle = wb.createCellStyle();

        toRow.setHeight(oldRow.getHeight());
        for (Iterator cellIt = oldRow.cellIterator(); cellIt.hasNext(); ) {
            Cell tmpCell = (Cell) cellIt.next();
            Cell newCell = toRow.createCell(tmpCell.getColumnIndex());
            copyCell(wb, tmpCell, newCell, newStyle);
        }
    }

    /**
     * 在同一个sheet内部进行行复制(多行)
     *
     * @param workbook     文档对象
     * @param sheet        sheet对象
     * @param fromRowIndex 从第n行开始复制
     * @param toRowIndex   粘贴到第m行
     * @param copyRowNum   复制行数
     */
    public static void copyRow(Workbook workbook, Sheet sheet, int fromRowIndex, int toRowIndex, int copyRowNum, boolean insertFlag
            , Integer colNum, CellStyle newStyle) {


        for (int i = 0; i < copyRowNum; i++) {
            Row fromRow = sheet.getRow(fromRowIndex + i);
            int currentToRowIndex = toRowIndex + i;
            Row toRow = sheet.getRow(currentToRowIndex);
            if (insertFlag) {
                //复制行超出原有sheet页的最大行数时，不需要移动直接插入
                if (currentToRowIndex <= sheet.getLastRowNum()) {
                    //先移动要插入的行号所在行及之后的行
                    sheet.shiftRows(currentToRowIndex, sheet.getLastRowNum(), 1, true, false);
                    //移动原有的形状位置（需要移动行里面的形状）
                    XSSFDrawing drawing = (XSSFDrawing) sheet.getDrawingPatriarch();
                    if (drawing != null) {
                        List<XSSFShape> shapeList = drawing.getShapes();
                        shapeList = !CollectionUtils.isEmpty(shapeList) ? shapeList.stream().filter(x ->
                                x.getAnchor() != null && ((XSSFClientAnchor) x.getAnchor()).getRow1() >= currentToRowIndex).collect(Collectors.toList()) : new ArrayList<>();
                        for (XSSFShape shape : shapeList) {
                            XSSFClientAnchor anchor = (XSSFClientAnchor) shape.getAnchor();
                            anchor.setRow1(anchor.getRow1() + 1);
                            anchor.setRow2(anchor.getRow2() + 1);
                        }
                    }
                }
                //然后再插入行
                toRow = sheet.createRow(currentToRowIndex);
                //设置行高
                toRow.setHeight(fromRow.getHeight());
            } else if(toRow == null) {
                //然后再插入行
                toRow = sheet.createRow(currentToRowIndex);
                //设置行高
                toRow.setHeight(fromRow.getHeight());
            }
            for (int colIndex = 0; colIndex < (colNum != null ? colNum : fromRow.getLastCellNum()); colIndex++) {
                Cell tmpCell = fromRow.getCell(colIndex);
                if (tmpCell == null) {
                    tmpCell = fromRow.createCell(colIndex);
                    if (colIndex != 0) {
                        copyCell(workbook, fromRow.createCell(colIndex - 1), tmpCell, newStyle);
                    }
                }
                Cell newCell = toRow.createCell(colIndex);
                copyCell(workbook, tmpCell, newCell, newStyle);
            }
        }
        //获取合并单元格
        List<CellRangeAddress> cellRangeAddressList = sheet.getMergedRegions();
        Map<Integer, List<CellRangeAddress>> rowCellRangeAddressMap = !CollectionUtils.isEmpty(cellRangeAddressList) ? cellRangeAddressList.stream().collect(Collectors.groupingBy(x ->
                x.getFirstRow())) : new HashMap<>();
        //获取形状（线条）
        XSSFDrawing drawing = (XSSFDrawing) sheet.getDrawingPatriarch();
        List<XSSFShape> shapeList = new ArrayList<>();
        Map<Integer, List<XSSFShape>> rowShapeMap = new HashMap<>();
        if (drawing != null) {
            shapeList = drawing.getShapes();
            rowShapeMap = shapeList.stream().filter(x -> x.getAnchor() != null)
                    .collect(Collectors.groupingBy(x -> ((XSSFClientAnchor) x.getAnchor()).getRow1()));
        }
        for (int i = 0; i < copyRowNum; i++) {
            Row toRow = sheet.getRow(toRowIndex + i);
            //复制合并单元格
            List<CellRangeAddress> rowCellRangeAddressList = rowCellRangeAddressMap.get(fromRowIndex + i);
            if (!CollectionUtils.isEmpty(rowCellRangeAddressList)) {
                for (CellRangeAddress cellRangeAddress : rowCellRangeAddressList) {
                    CellRangeAddress newCellRangeAddress = new CellRangeAddress(toRow.getRowNum(), (toRow.getRowNum() +
                            (cellRangeAddress.getLastRow() - cellRangeAddress.getFirstRow())), cellRangeAddress
                            .getFirstColumn(), cellRangeAddress.getLastColumn());
                    sheet.addMergedRegionUnsafe(newCellRangeAddress);
                }
            }
            //复制形状（线条）
            List<XSSFShape> rowShapeList = rowShapeMap.get(fromRowIndex + i);
            if (!CollectionUtils.isEmpty(rowShapeList)) {
                for (XSSFShape shape : rowShapeList) {
                    //复制描点
                    XSSFClientAnchor fromAnchor = (XSSFClientAnchor) shape.getAnchor();
                    XSSFClientAnchor toAnchor = new XSSFClientAnchor();
                    toAnchor.setDx1(fromAnchor.getDx1());
                    toAnchor.setDx2(fromAnchor.getDx2());
                    toAnchor.setDy1(fromAnchor.getDy1());
                    toAnchor.setDy2(fromAnchor.getDy2());
                    toAnchor.setRow1(toRow.getRowNum());
                    toAnchor.setRow2(toRow.getRowNum() + fromAnchor.getRow2() - fromAnchor.getRow1());
                    toAnchor.setCol1(fromAnchor.getCol1());
                    toAnchor.setCol2(fromAnchor.getCol2());
                    //复制形状
                    if (shape instanceof XSSFConnector) {
                        copyXSSFConnector((XSSFConnector) shape, drawing, toAnchor);
                    } else if (shape instanceof XSSFSimpleShape) {
                        copyXSSFSimpleShape((XSSFSimpleShape) shape, drawing, toAnchor);
                    }
                }
            }
        }
    }

    /**
     * 复制XSSFSimpleShape类
     *
     * @param fromShape
     * @param drawing
     * @param anchor
     * @return
     */
    public static XSSFSimpleShape copyXSSFSimpleShape(XSSFSimpleShape fromShape, XSSFDrawing drawing, XSSFClientAnchor anchor) {
        XSSFSimpleShape toShape = drawing.createSimpleShape(anchor);
        CTShape ctShape = fromShape.getCTShape();
        CTShapeProperties ctShapeProperties = ctShape.getSpPr();
        CTLineProperties lineProperties = ctShapeProperties.isSetLn() ? ctShapeProperties.getLn() : ctShapeProperties.addNewLn();
        CTPresetLineDashProperties dashStyle = lineProperties.isSetPrstDash() ? lineProperties.getPrstDash() : CTPresetLineDashProperties.Factory.newInstance();
        STPresetLineDashVal.Enum dashStyleEnum = dashStyle.isSetVal() ? dashStyle.getVal() : STPresetLineDashVal.Enum.forInt(1);
        CTSolidColorFillProperties fill = lineProperties.isSetSolidFill() ? lineProperties.getSolidFill() : lineProperties.addNewSolidFill();
        CTSRgbColor rgb = fill.isSetSrgbClr() ? fill.getSrgbClr() : CTSRgbColor.Factory.newInstance();
        // 设置形状类型
        toShape.setShapeType(fromShape.getShapeType());
        // 设置线宽
        toShape.setLineWidth(lineProperties.getW() * 1.0 / Units.EMU_PER_POINT);
        // 设置线的风格
        toShape.setLineStyle(dashStyleEnum.intValue() - 1);
        // 设置线的颜色
        byte[] rgbBytes = rgb.getVal();
        if (rgbBytes == null) {
            toShape.setLineStyleColor(0, 0, 0);
        } else {
            toShape.setLineStyleColor(rgbBytes[0], rgbBytes[1], rgbBytes[2]);
        }
        return toShape;
    }

    /**
     * 复制XSSFConnector类
     *
     * @param fromShape
     * @param drawing
     * @param anchor
     * @return
     */
    public static XSSFConnector copyXSSFConnector(XSSFConnector fromShape, XSSFDrawing drawing, XSSFClientAnchor anchor) {
        XSSFConnector toShape = drawing.createConnector(anchor);
        CTConnector ctConnector = fromShape.getCTConnector();
        CTShapeProperties ctShapeProperties = ctConnector.getSpPr();
        CTLineProperties lineProperties = ctShapeProperties.isSetLn() ? ctShapeProperties.getLn() : ctShapeProperties.addNewLn();
        CTPresetLineDashProperties dashStyle = lineProperties.isSetPrstDash() ? lineProperties.getPrstDash() : CTPresetLineDashProperties.Factory.newInstance();
        STPresetLineDashVal.Enum dashStyleEnum = dashStyle.isSetVal() ? dashStyle.getVal() : STPresetLineDashVal.Enum.forInt(1);
        CTSolidColorFillProperties fill = lineProperties.isSetSolidFill() ? lineProperties.getSolidFill() : lineProperties.addNewSolidFill();
        CTSRgbColor rgb = fill.isSetSrgbClr() ? fill.getSrgbClr() : CTSRgbColor.Factory.newInstance();
        // 设置形状类型
        toShape.setShapeType(fromShape.getShapeType());
        // 设置线宽
        toShape.setLineWidth(lineProperties.getW() * 1.0 / Units.EMU_PER_POINT);
        // 设置线的风格
        toShape.setLineStyle(dashStyleEnum.intValue() - 1);
        // 设置线的颜色
        byte[] rgbBytes = rgb.getVal();
        if (rgbBytes == null) {
            toShape.setLineStyleColor(0, 0, 0);
        } else {
            toShape.setLineStyleColor(rgbBytes[0], rgbBytes[1], rgbBytes[2]);
        }
        return toShape;
    }

    /**
     * Sheet复制
     *
     * @param wb        workbook
     * @param fromSheet 要复制的Sheet
     * @Parame toSheet 复制到的Sheet
     */
    public static void copySheet(Workbook wb, Sheet fromSheet, Sheet toSheet) {
        mergeSheetAllRegion(fromSheet, toSheet);
        // 设置列宽
        int length = fromSheet.getRow(fromSheet.getFirstRowNum()).getLastCellNum();
        for (int i = 0; i <= length; i++) {
            toSheet.setColumnWidth(i, fromSheet.getColumnWidth(i));
        }
        for (Iterator rowIt = fromSheet.rowIterator(); rowIt.hasNext(); ) {
            Row oldRow = (Row) rowIt.next();
            Row newRow = toSheet.createRow(oldRow.getRowNum());
            copyRow(wb, oldRow, newRow);
        }
    }

    /**
     * 合并单元格
     *
     * @param fromSheet
     * @param toSheet
     */
    private static void mergeSheetAllRegion(Sheet fromSheet, Sheet toSheet) {
        int num = fromSheet.getNumMergedRegions();
        CellRangeAddress cellR = null;
        for (int i = 0; i < num; i++) {
            cellR = fromSheet.getMergedRegion(i);
            toSheet.addMergedRegion(cellR);
        }
    }

    /**
     * 复制单元格
     *
     * @param srcCell
     * @param distCell
     */
    public static void copyCell(Workbook workbook, Cell srcCell, Cell distCell, CellStyle newStyle) {
        // 克隆style
        newStyle.cloneStyleFrom(srcCell.getCellStyle());
        //样式
        distCell.setCellStyle(newStyle);
        //设置内容
        CellType srcCellType = srcCell.getCellType();
        //distCell.setCellFormula(srcCell.getCellFormula());
        if (srcCellType == NUMERIC) {
            if (DateUtil.isCellDateFormatted(srcCell)) {
                distCell.setCellValue(srcCell.getDateCellValue());
            } else {
                distCell.setCellValue(srcCell.getNumericCellValue());
            }
        } else if (srcCellType == STRING) {
            distCell.setCellValue(srcCell.getRichStringCellValue());
        } else if (srcCellType == CellType.BOOLEAN) {
            distCell.setCellValue(srcCell.getBooleanCellValue());
        } else if (srcCellType == CellType.ERROR) {
            distCell.setCellErrorValue(srcCell.getErrorCellValue());
        } else if (srcCellType == FORMULA) {
            distCell.setCellFormula(srcCell.getCellFormula());
        } else {
        }
    }

    /**
     * 得到RBG自定义颜色
     *
     * @param redNum   红色数值
     * @param greenNum 绿色数值
     * @param blueNum  蓝色数值
     * @return
     */
    public static XSSFColor getXSSFRGBColor(int redNum, int greenNum, int blueNum) {
        XSSFColor color = new XSSFColor(new byte[]{(byte) redNum, (byte) greenNum, (byte) blueNum}, new DefaultIndexedColorMap());
        return color;
    }


    /**
     * 合并单元格并处理重叠，避免递归和无限循环。
     *
     * @param sheet       工作表
     * @param firstRow    开始行
     * @param lastRow     结束行
     * @param firstCol    开始列
     * @param lastCol     结束列
     */
    public static void mergeCellsAvoidOverlap(Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol) throws IllegalArgumentException {
        int maxRetry = 5;
        for (int retry = 0; retry < maxRetry; retry++) {
            // 先尝试取消所有与即将合并区域有交集的已存在合并区域
            cancelOverlappedMergedRegions(sheet, firstRow, lastRow, firstCol, lastCol);

            try {
                sheet.addMergedRegion(new CellRangeAddress(firstRow, lastRow, firstCol, lastCol));
                // 成功合并后跳出循环
                break;
            } catch (IllegalStateException ise) {
                if (retry == maxRetry - 1) {
                    // 达到最大重试次数仍失败，抛出异常
                    throw new IllegalArgumentException("尝试合并单元格 [" + firstRow + "," + firstCol + "] 至 [" + lastRow + "," + lastCol + "] 失败，可能因为重叠问题未解决。", ise);
                }
                // 记录日志或处理逻辑
                log.warn("合并单元格时遇到重叠问题，正在进行第{}次重试。", retry + 1, ise);
            }
        }
    }



    /**
     * 取消与指定区域有交集的所有已合并区域。
     *
     * @param sheet       工作表
     * @param firstRow    新合并区域的开始行
     * @param lastRow     新合并区域的结束行
     * @param firstCol    新合并区域的开始列
     * @param lastCol     新合并区域的结束列
     */
    private static void cancelOverlappedMergedRegions(Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol) {
        List<CellRangeAddress> regionsToRemove = new ArrayList<>();
        for (int i = 0; i < sheet.getNumMergedRegions(); i++) {
            CellRangeAddress existingRegion = sheet.getMergedRegion(i);
            if (regionsOverlap(existingRegion, firstRow, lastRow, firstCol, lastCol)) {
                regionsToRemove.add(existingRegion);
            }
        }

        // 逆序移除，避免索引错位
        Collections.reverse(regionsToRemove);
        for (int i = regionsToRemove.size() - 1; i >= 0; i--) {
            CellRangeAddress region = regionsToRemove.get(i);
            int index = sheet.getMergedRegions().indexOf(region);
            if (index != -1) {
                sheet.removeMergedRegion(index);
            }
        }
    }


    /**
     * 判断两个区域是否有交集。
     *
     * @param region1  区域1
     * @param startRow2 区域2起始行
     * @param endRow2   区域2结束行
     * @param startCol2 区域2起始列
     * @param endCol2   区域2结束列
     * @return 是否有交集
     */
    private static boolean regionsOverlap(CellRangeAddress region1, int startRow2, int endRow2, int startCol2, int endCol2) {
        return !((region1.getLastRow() < startRow2) || (region1.getFirstRow() > endRow2) ||
                (region1.getLastColumn() < startCol2) || (region1.getFirstColumn() > endCol2));
    }



//    /**
//     * 合并单元格
//     * @param sheet
//     * @param firstRow
//     * @param lastRow
//     * @param firstCol
//     * @param lastCol
//     */
//    public static void addMergedRegion(Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol) {
//        int retryLimit = 5; // 设定最大重试次数为5 防止栈溢出
//        addMergedRegionWithRetry(sheet, firstRow, lastRow, firstCol, lastCol, retryLimit);
//
//    }
//
//    private static void addMergedRegionWithRetry(Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol, int retryLimit) {
//
//        try {
//            sheet.addMergedRegion(new CellRangeAddress(firstRow, lastRow, firstCol, lastCol));
//        } catch (IllegalStateException var6){
//            if (retryLimit > 0){
//                //合并单元格的循环 可能存在 已经合并的再次合并  不管他
//                cancelMerged(sheet, firstRow, lastRow, firstCol, lastCol);
//                addMergedRegionWithRetry(sheet, firstRow, lastRow, firstCol, lastCol, retryLimit - 1);
//            }else {
//                // 达到重试次数上限，记录错误或抛出异常
//                log.error("合并单元格重试达到上限，无法完成合并操作。", var6);
//            }
//
//        } catch (Exception var6) {
//            log.debug("发生了一次合并单元格错误,{},{},{},{}", firstRow, lastRow, firstCol, lastCol);
//            log.debug(var6.getMessage(), var6);
//        }
//    }
//
//
//    // 取消指定区域的合并单元格
//    private static void cancelMerged(Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol) {
//        for (int i = sheet.getNumMergedRegions() - 1; i >= 0; i--) {
//            CellRangeAddress mergedRegion = sheet.getMergedRegion(i);
//            if (mergedRegion.isInRange(firstRow, firstCol) && mergedRegion.isInRange(lastRow, lastCol)) {
//                sheet.removeMergedRegion(i);
//            }
//        }
//    }

    /**
     * 取消某一个范围内的合并单元格
     *
     * @param sheet
     * @param firstRow
     * @param lastRow
     * @param firstCol
     * @param lastCol
     */
    public static void cancelMerged(Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol) {
        List<CellRangeAddress> listRange = sheet.getMergedRegions();

        List<Integer> removeRangeNums = new ArrayList<Integer>();
        CellRangeAddress r = null;
        for (int i = 0; i < listRange.size(); i++) {
            r = listRange.get(i);
            if (r.getFirstRow() >= firstRow && r.getLastRow() <= lastRow && r.getFirstColumn() >= firstCol && r.getLastColumn() <= lastCol) {
                removeRangeNums.add(i);
            }
        }
        sheet.removeMergedRegions(removeRangeNums);
    }

    /**
     * 删除行
     *
     * @param sheet
     * @param startRow
     * @param endRow
     * @return
     */
    public static Sheet deleteRow(Sheet sheet, int startRow, int endRow) {
        //获取最大的列数
        int maxColNum = 0;
        Iterator<Row> rowIt = sheet.rowIterator();
        while (rowIt.hasNext()) {
            maxColNum = Math.max(maxColNum, rowIt.next().getLastCellNum());
        }

        //向下移动
        int startNumDown = endRow + 1;
        int endNumDown = sheet.getLastRowNum();
        int moveNumDown = endNumDown - startNumDown + 1;
        sheet.shiftRows(startNumDown, endNumDown, moveNumDown, true, true);
        //图片的处理
        XSSFDrawing drawings = (XSSFDrawing) sheet.createDrawingPatriarch();
        for (XSSFShape shape : drawings.getShapes()) {
            if (shape instanceof Picture) {
                XSSFPicture picture = (XSSFPicture) shape;
                XSSFClientAnchor anchor = picture.getClientAnchor();
                int row1 = anchor.getRow1();
                if (row1 >= startNumDown && row1 <= endNumDown) {
                    int row2 = anchor.getRow2();
                    anchor.setRow1(row1 + moveNumDown);
                    anchor.setRow2(row2 + moveNumDown);
                }
            }
        }

        //空白部分 取消合并处理
        cancelMerged(sheet, startRow, endRow + moveNumDown, 0, maxColNum);
        //重置这一部分
        for (int i = startRow; i < endRow + moveNumDown; i++) {
            sheet.createRow(i);
            sheet.removeRow(sheet.getRow(i));
        }

        //再向上移动回去
        int startNumUp = endNumDown + 1;
        int endNumUp = sheet.getLastRowNum();
        int moveNumUp = moveNumDown + (endRow - startRow) + 1;
        sheet.shiftRows(startNumUp, endNumUp, -1 * moveNumUp, true, true);
        //图片的处理
        drawings = (XSSFDrawing) sheet.createDrawingPatriarch();
        for (XSSFShape shape : drawings.getShapes()) {
            if (shape instanceof Picture) {
                XSSFPicture picture = (XSSFPicture) shape;
                XSSFClientAnchor anchor = picture.getClientAnchor();
                int row1 = anchor.getRow1();
                if (row1 >= startNumUp && row1 <= endNumUp) {
                    int row2 = anchor.getRow2();
                    anchor.setRow1(row1 - moveNumUp);
                    anchor.setRow2(row2 - moveNumUp);
                }
            }
        }

        return sheet;
    }

    /**
     * 复制列
     *
     * @param currentSheet
     * @param pStartCol
     * @param pEndCol
     * @param pPosition    相对于pStartCol的偏移量
     */
    public static void copyCols(Sheet currentSheet, int pStartCol, int pEndCol, int pPosition) {
        Workbook workbook = currentSheet.getWorkbook();
        CellStyle cellStyle = workbook.createCellStyle();
        int targetColFrom;
        int targetColTo;
        CellRangeAddress region = null;
        int i;
        int j;
        if (pStartCol == -1 || pEndCol == -1) {
            return;
        }
        //System.out.println(currentSheet.getNumMergedRegions());
        for (i = 0; i < currentSheet.getNumMergedRegions(); i++) {
            region = currentSheet.getMergedRegion(i);
            if ((region.getFirstColumn() >= pStartCol)
                    && (region.getLastColumn() <= pEndCol)) {
                targetColFrom = region.getFirstColumn() - pStartCol + pPosition;
                targetColTo = region.getLastColumn() - pStartCol + pPosition;
                CellRangeAddress newRegion = region.copy();
                newRegion.setFirstRow(region.getFirstRow());
                newRegion.setFirstColumn(targetColFrom);
                newRegion.setLastRow(region.getLastRow());
                newRegion.setLastColumn(targetColTo);
                currentSheet.addMergedRegion(newRegion);

            }
        }
        for (i = 0; i <= 50; i++) {
            Row sourceRow = currentSheet.getRow(i);
            if (sourceRow != null) {
                Row newRow = currentSheet.getRow(i);
                for (j = pStartCol; j <= pEndCol; j++) {
                    Cell templateCell = sourceRow.getCell(j);
                    if (i == 0) {
                        currentSheet.setColumnWidth(pPosition + j, currentSheet.getColumnWidth(j));
                    }
                    if (templateCell != null) {
                        Cell newCell = newRow.createCell(pPosition + j);
                        copyCell(currentSheet.getWorkbook(), templateCell, newCell, cellStyle);
                    }
                }
            }
        }
    }


    /**
     * 自适应excel行高
     *
     * @param cell   列
     * @param margin 字体边距
     */

    public static void autoRowHeight(Cell cell, String cellVal, int margin) {
        if (cell == null) {
            return;
        }
        if (StringUtils.isBlank(cellVal)) {
            return;
        }
        Row row = cell.getRow();
        Workbook workbook = row.getSheet().getWorkbook();
        short charHeight = 0;
        if (workbook instanceof HSSFWorkbook) {
            charHeight = ((HSSFCell) cell).getCellStyle().getFont(workbook).getFontHeightInPoints();
        } else if (workbook instanceof XSSFWorkbook) {
            charHeight = ((XSSFCell) cell).getCellStyle().getFont().getFontHeightInPoints();
        }
        double charPx = charHeight * Units.PIXEL_DPI / Units.POINT_DPI;
        double cellWidthPx = getCellWidthInPixels(cell);//宽度像素

        String[] strs = cellVal.split("\n");
        int rowNum = 0;
        for (int i = 0; i < strs.length; i++) {
            int charLength = cellVal.length();//字符长度
            int charInCell = (int) (cellWidthPx / charPx);//每一个列（包括合并的列）的字数
            //在指定宽度的列中的字符展现行数
            rowNum += Math.max(charLength / charInCell + (charLength % charInCell > 0 ? 1 : 0), 1);
        }
        //自适应以后的行高点数
        short cellHeightPx = (short) (((short) rowNum) * (charHeight + margin));
        row.setHeightInPoints(cellHeightPx);
    }

    /**
     * 获取单元格及合并单元格的宽度
     *
     * @param cell 单元格
     * @return 单元格及合并单元格的宽度（像素）
     */
    public static double getCellWidthInPixels(Cell cell) {
        return getCellWidthInPixels(cell, null);
    }

    /**
     * 获取单元格及合并单元格的宽度
     *
     * @param cell 单元格
     * @return 单元格及合并单元格的宽度（像素）
     */
    public static double getCellWidthInPixels(Cell cell, CellRangeAddress range) {
        if (cell == null)
            return 0.0;
        if (range == null)
            range = getCellRangeAddress(cell);
        Sheet sheet = cell.getSheet();
        if (range != null) {
            int firstColumn = range.getFirstColumn();
            int lastColumn = range.getLastColumn();
            double width = 0.0;
            for (int i = firstColumn; i <= lastColumn; i++) {
                width += sheet.getColumnWidthInPixels(i);
            }
            return width;
        } else {
            return sheet.getColumnWidthInPixels(cell.getColumnIndex());
        }
    }

    /**
     * 获取单元格及合并单元格的宽度
     *
     * @param cell 单元格
     * @return 单元格及合并单元格的宽度（像素）
     */
    public static double getCellHeightInPixels(Cell cell) {
        return getCellHeightInPixels(cell, null);
    }

    /**
     * 获取单元格及合并单元格的宽度
     *
     * @param cell  单元格
     * @param range 单元格范围地址
     * @return 单元格及合并单元格的宽度（像素）
     */
    public static double getCellHeightInPixels(Cell cell, CellRangeAddress range) {
        if (cell == null)
            return 0.0;
        if (range == null)
            range = getCellRangeAddress(cell);
        Sheet sheet = cell.getSheet();
        if (range != null) {
            int firstRow = range.getFirstRow();
            int lastRow = range.getLastRow();
            double height = 0.0;
            for (int i = firstRow; i <= lastRow; i++) {
                height += sheet.getRow(i).getHeightInPoints() * Units.PIXEL_DPI / Units.POINT_DPI;
            }
            return height;
        } else {
            return cell.getRow().getHeightInPoints() * Units.PIXEL_DPI / Units.POINT_DPI;
        }
    }

    /**
     * 获取单元格所在范围地址
     *
     * @param cell 对象
     * @return 范围地址
     */
    public static CellRangeAddress getCellRangeAddress(Cell cell) {
        if (cell == null) {
            return null;
        }
        Sheet sheet = cell.getSheet();
        int sheetMergeCount = sheet.getNumMergedRegions();
        for (int i = 0; i < sheetMergeCount; i++) {
            CellRangeAddress ca = sheet.getMergedRegion(i);
            if (ca.isInRange(cell)) {
                return ca;
            }
        }
        return null;
    }

    public static Cell getCell(Sheet sheet, String excelPos) {
        ExcelTool.ExcelPos pos = ExcelTool.convertExcelPos(excelPos);
        return sheet.getRow(pos.rowIndex).getCell(pos.colIndex);
    }

    /**
     * @Description：根据文件后缀，自适应上传文件的版本
     * @param inStr,fileName
     * @return
     * @throws Exception
     */
    public static Workbook getWorkbook(InputStream inStr, String fileName) {
        Workbook wb = null;
        String fileType = fileName.substring(fileName.lastIndexOf("."));
        ZipSecureFile.setMinInflateRatio(-1.0d);
        try {
            if (".xls".equals(fileType)) {
                wb = new HSSFWorkbook(inStr);  //2003-
            } else if (".xlsx".equals(fileType)) {
                wb = new XSSFWorkbook(inStr);  //2007+
            }
        } catch (IOException e) {
            throw new BaseException("解析失败，"+e.getMessage());
        }

        if(wb == null) {
            throw new BaseException("解析的文件格式有误！");
        }

        return wb;
    }

    public static void setCellValue(Cell cell, Object value) {
        if(cell == null) return;
        if(value == null) return;
        CellType cellType = cell.getCellType();
        if(cellType == NUMERIC) {
            if (DateUtil.isCellDateFormatted(cell)) {
                if(value instanceof Date) {
                    cell.setCellValue((Date) value);
                } else if(value instanceof String){
                    Date d = DateUtils.parseDate(value);
                    if(d != null) {
                        cell.setCellValue(d);
                    }
                } else if(value instanceof Number) {
                    cell.setCellValue(DateUtil.getJavaDate(Double.parseDouble(value.toString())));
                }
            } else {
                if(!(value instanceof Double)) {
                    Assert.isTrue(value.toString().matches("^(-?\\d+)(\\.\\d+)?$"), "类型不匹配");
                    value = Double.parseDouble(value.toString());
                }
                cell.setCellValue((Double) value);
            }
        } else if(cellType == STRING || cellType == FORMULA ) {
            if(value instanceof Date) {
                cell.setCellValue((Date) value);
            } else {
                Date d = DateUtils.parseDate(value);
                if(d != null) {
                    cell.setCellValue(d);
                } else {
                    cell.setCellValue(value.toString());
                }
            }
        } else if(cellType == BOOLEAN) {
            Assert.isTrue(value instanceof Boolean, "类型不匹配");
            cell.setCellValue((Boolean)value);
        } else if(cellType == BLANK) {
            if(value instanceof Date) {
                cell.setCellValue((Date)value);
            } else if(value instanceof Calendar) {
                cell.setCellValue((Calendar)value);
            } else if(value instanceof LocalDateTime) {
                cell.setCellValue((LocalDateTime)value);
            } else if(value instanceof LocalDate) {
                cell.setCellValue((LocalDate)value);
            } else if(value instanceof BigDecimal) {
                cell.setCellValue(Double.parseDouble(value.toString()));
            } else {
                cell.setCellValue(value.toString());
            }
        }
    }

    /**
     * 获取图片和位置 (xls)
     * @param sheet
     * @return
     */
    public static Map<String, HSSFPictureData> getPictures (HSSFSheet sheet) {
        Map<String, HSSFPictureData> map = new HashMap<>();
        List<HSSFShape> list = sheet.getDrawingPatriarch().getChildren();
        for (HSSFShape shape : list) {
            if (shape instanceof HSSFPicture) {
                HSSFPicture picture = (HSSFPicture) shape;
                HSSFClientAnchor cAnchor = picture.getClientAnchor();
                HSSFPictureData pdata = picture.getPictureData();
                String key = cAnchor.getRow1() + ":" + cAnchor.getCol1(); // 行号:列号 （都基于0）
                map.put(key, pdata);
            }
        }
        return map;
    }

    /**
     * 获取图片和位置 (xlsx)
     * @param sheet
     * @return
     */
    public static Map<String, XSSFPictureData> getPictures (XSSFSheet sheet) {
        Map<String, XSSFPictureData> map = new HashMap<>();
        List<POIXMLDocumentPart> list = sheet.getRelations();
        for (POIXMLDocumentPart part : list) {
            if (part instanceof XSSFDrawing) {
                XSSFDrawing drawing = (XSSFDrawing) part;
                List<XSSFShape> shapes = drawing.getShapes();
                for (XSSFShape shape : shapes) {
                    try {
                        XSSFPicture picture = (XSSFPicture) shape;
                        XSSFClientAnchor anchor = picture.getPreferredSize();
                        CTMarker marker = anchor.getFrom();
                        String key = marker.getRow() + ":" + marker.getCol(); // 行号:列号 （都基于0）
                        map.put(key, picture.getPictureData());
                    }catch (Exception e){
                        throw new BaseException("图片类型不支持解析,获取失败!");
                    }
                }
            }
        }
        return map;
    }

    /**
     * 获取图片和位置
     * @param sheet
     * @return
     */
    public static Map<String, PictureData> getPictures (Sheet sheet) {
        Map<String, PictureData> retma = new HashMap<>();
        if(sheet instanceof XSSFSheet) { // excel 2007+
            Map<String, XSSFPictureData> map = getPictures((XSSFSheet) sheet);
            map.forEach((key,data)->{
                retma.put(key, data);
            });
        } else if(sheet instanceof HSSFSheet) {// excel 2003
            Map<String, HSSFPictureData> map = getPictures((HSSFSheet) sheet);
            map.forEach((key,data)->{
                retma.put(key, data);
            });
        }
        return retma;
    }

    /**
     *  设置Excel单元格为数值类型便于公式运算
     *
     * @param workbook
     * @param contextstyle
     * @param cell
     * @param data
     */
    public static void setNumberFormat(Workbook workbook, CellStyle contextstyle, Cell cell, Object data){
        boolean isNum = false;//data是否为数值型
        boolean isInteger=false;//data是否为整数
        boolean isPercent=false;//data是否为百分数
        if (data != null || "".equals(data)) {
            //判断data是否为数值型
            isNum = data.toString().matches("^(-?\\d+)(\\.\\d+)?$");
            //判断data是否为整数（小数部分是否为0）
            isInteger=data.toString().matches("^[-\\+]?[\\d]*$");
            //判断data是否为百分数（是否包含“%”）
            isPercent=data.toString().contains("%");
        }

        //如果单元格内容是数值类型，设置data的类型为数值类型
        if (isNum && !isPercent) {
            // 此处设置数据格式
            DataFormat df = workbook.createDataFormat();
            if (isInteger) {
                contextstyle.setDataFormat(df.getFormat("0_ "));//数据格式只显示整数
            }else{
                contextstyle.setDataFormat(df.getFormat("0.00_ "));//保留两位小数点
            }
            // 设置单元格格式
            cell.setCellStyle(contextstyle);
            // 设置单元格内容为double类型
            cell.setCellValue(Double.parseDouble(data.toString()));
        } else {
            if (data != null && !"".equals(data)) {
                cell.setCellStyle(contextstyle);
                // 设置单元格内容为字符型
                cell.setCellValue(data.toString());
            }
        }
    }

    /**
     * 插入图片，图片填充表格，不变形，居中或左上角
     *
     * @param workbook 工作簿
     * @param sheet    工作表
     * @param imageUrl 图片地址
     * @param row      开始行
     * @param col      开始列
     */
    public static void insertUrlImageToCell(Workbook workbook, Sheet sheet, String imageUrl, boolean center, int row, int col) {
        int space = 2; // 边距
        try {
            byte[] bytes = HttpUtil.downloadBytes(imageUrl);
            Cell cell = sheet.getRow(row).getCell(col);
            CellRangeAddress range = getCellRangeAddress(cell);
            double cellWidth = getCellWidthInPixels(cell, range); // 单元格宽度（像素）
            double cellHeight = getCellHeightInPixels(cell, range); // 单元格高度（像素）
            int endRow = row + 1;
            int endCol = col + 1;
            if (range != null) {
                endRow = range.getLastRow() + 1;
                endCol = range.getLastColumn() + 1;
            }

            Drawing drawing = sheet.getDrawingPatriarch();
            if (drawing == null) {
                drawing = sheet.createDrawingPatriarch();
            }
            ClientAnchor anchor = null;
            if (workbook instanceof HSSFWorkbook) {
                anchor = new HSSFClientAnchor(0, 0, 0, 0, (short) col, row, (short) endCol, endRow);
            } else if (workbook instanceof XSSFWorkbook) {
                anchor = new XSSFClientAnchor(0, 0, 0, 0, (short) col, row, (short) endCol, endRow);
            }
            anchor.setAnchorType(ClientAnchor.AnchorType.MOVE_DONT_RESIZE);
            int picIndex = workbook.addPicture(bytes, Workbook.PICTURE_TYPE_PNG);
            Picture picture = drawing.createPicture(anchor, picIndex);
            picture.resize(1.0); // 重置尺寸到填充表格

            Dimension dimension = picture.getImageDimension();
            double targetRate = (cellWidth - space * 2.0) / dimension.getWidth(); // 宽度缩小比例
            double targetHeight = picture.getImageDimension().getHeight() * targetRate; // 缩小后的高度
            if (targetHeight > (cellHeight - space * 2.0)) { // 缩小后的高度是否在单元格内
                targetRate = (cellHeight - space * 2.0) / dimension.getHeight(); // 高度缩小比例
            }
            if(targetRate>1){
                targetRate = 1;
            }
            double targetWidth = dimension.getWidth() * targetRate; // 目标宽度
            targetHeight = dimension.getHeight() * targetRate; // 目标高度

            if (center) {
                double posx = (cellWidth - targetWidth) / 2.0; // 目标居中x坐标
                double posy = (cellHeight - targetHeight) / 2.0; // 目标居中y坐标
                double posx2 = posx;
                double posy2 = posy;
                if (endCol == col + 1) {
                    anchor.setDx1((int) posx * Units.EMU_PER_PIXEL);
                    anchor.setDx2(-(int) posx2 * Units.EMU_PER_PIXEL);
                } else {
                    double width = 0;
                    for (int i = col; i < endCol; i++) {
                        double beforeWidtth = width;
                        width += sheet.getColumnWidthInPixels(i);
                        if (posx >= (int) beforeWidtth && posx < (int) width) {
                            posx = posx - (int) beforeWidtth;
                            anchor.setCol1(i);
                            anchor.setCol2(endCol);
                            anchor.setDx1((int) posx * Units.EMU_PER_PIXEL);
                            anchor.setDx2(-(int) posx2 * Units.EMU_PER_PIXEL);
                            break;
                        }
                    }
                }
                if (endRow == row + 1) {
                    anchor.setDy1((int) posy * Units.EMU_PER_PIXEL);
                    anchor.setDy2(-(int) posy2 * Units.EMU_PER_PIXEL);
                } else {
                    double height = 0;
                    for (int i = row; i < endRow; i++) {
                        double beforeHeight = height;
                        height += sheet.getRow(i).getHeightInPoints() * Units.PIXEL_DPI / Units.POINT_DPI;
                        if (posy >= (int) beforeHeight && posy < (int) height) {
                            posy = posy - (int) beforeHeight;
                            anchor.setRow1(i);
                            anchor.setRow2(endRow);
                            anchor.setDy1((int) posy * Units.EMU_PER_PIXEL);
                            anchor.setDy2(-(int) posy2 * Units.EMU_PER_PIXEL);
                            break;
                        }
                    }
                }
            } else {
                double posx = space;
                double posy = space;
                double posx2 = cellWidth - targetWidth - space * 2.0;
                double posy2 = cellHeight - targetHeight - space * 2.0;
                anchor.setDx1((int) posx * Units.EMU_PER_PIXEL);
                anchor.setDy1((int) posy * Units.EMU_PER_PIXEL);
                anchor.setDx2(-(int) posx2 * Units.EMU_PER_PIXEL);
                anchor.setDy2(-(int) posy2 * Units.EMU_PER_PIXEL);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
